Array.prototype.makeIterable = function () {
    return AH.makeIterable(this);
};
var AhValidation;
(function (AhValidation) {
    var ValidationType;
    (function (ValidationType) {
        ValidationType[ValidationType["None"] = 0] = "None";
        ValidationType[ValidationType["Integer"] = 1] = "Integer";
        ValidationType[ValidationType["Double"] = 2] = "Double";
        ValidationType[ValidationType["Date"] = 3] = "Date";
        ValidationType[ValidationType["Boolean"] = 4] = "Boolean";
    })(ValidationType = AhValidation.ValidationType || (AhValidation.ValidationType = {}));
    function unwrap(value) {
        if (typeof value === 'function') {
            return value();
        }
        else {
            return value;
        }
    }
    ;
    /**
     * Validates an individual field
     * @param validator to apply
     * @param value field to apply to
     */
    function ValidateField(validator, value) {
        var fieldName = unwrap(validator.FieldName) || 'Field';
        var message = unwrap(validator.Message);
        if (!value && unwrap(validator.Required)) {
            return { IsValid: false, Message: fieldName + ' is required.' };
        }
        else if (value) {
            if (validator.MaximumLength && (value + '').length > unwrap(validator.MaximumLength)) {
                return { IsValid: false, Message: message || (fieldName + ' exceeds maximum length of ' + unwrap(validator.MaximumLength) + '.') };
            }
            if (unwrap(validator.Type) == ValidationType.Integer && !/^[0-9]+$/.test(value)) {
                return { IsValid: false, Message: message || (fieldName + ' must be an integer.') };
            }
            if (unwrap(validator.Type) == ValidationType.Double && !/^[0-9\.]+$/.test(value)) {
                return { IsValid: false, Message: message || (fieldName + ' must be a number.') };
            }
            if (validator.Pattern && !(new RegExp(unwrap(validator.Pattern))).test(value)) {
                return { IsValid: false, Message: message || (fieldName + ' has an invalid format.') };
            }
        }
        return { IsValid: true };
    }
    AhValidation.ValidateField = ValidateField;
    /**
     * Validates all elements on a form, optionally applying the results as an attribute
     * @param form element to search for validators under
     * @param applyResults when set, adds a 'ah-validationresults' attribute to the element
     */
    function ValidateForm(form, applyResults) {
        if (applyResults === void 0) { applyResults = false; }
        var results = [];
        $('[data-ahvalidator]', form).each(function (idx, el) {
            var validator = JSON.parse(el.getAttribute('data-ahvalidator'));
            if (!validator)
                return;
            var result = AhValidation.ValidateField(validator, $(el).val());
            results.push(result);
            if (applyResults) {
                el.setAttribute('data-ahvalidationresults', JSON.stringify(result));
            }
        });
        return results;
    }
    AhValidation.ValidateForm = ValidateForm;
    /**
     * Applies validators to all elements in all forms
     */
    function InitializeElement(el) {
        var initialResults = JSON.parse(el.getAttribute('data-ahvalidationresults'));
        var blurLocked = initialResults && !initialResults.IsValid;
        AhValidation.DisplayResult(el);
        $(el).change(function (e) { blurLocked = false; }); // only remove server-set validation if changed
        $(el).blur(function (e) {
            if (blurLocked)
                return;
            var result = AhValidation.ValidateField(JSON.parse(el.getAttribute('data-ahvalidator')), $(el).val());
            el.setAttribute('data-ahvalidationresults', JSON.stringify(result));
            AhValidation.DisplayResult(el);
        });
    }
    AhValidation.InitializeElement = InitializeElement;
    /**
     * Applies validators to all elements in all forms
     */
    function InitializeForms() {
        for (var i = 0; i < document.forms.length; i++) {
            (function (form) {
                $('[data-ahvalidator]', form).each(function (idx, el) { return InitializeElement(el); });
            })(document.forms[i]);
        }
    }
    AhValidation.InitializeForms = InitializeForms;
    /**
     * Validates and submits a form from a specified button and applies the args as hidden element
     * @param button
     * @param args
     */
    function Submit(button, args) {
        function findForm(el) {
            if (el.parentElement.nodeName.toLowerCase() == 'form')
                return el.parentElement;
            else
                return findForm(el.parentElement);
        }
        var form = findForm(button);
        if (form == null)
            throw new Error('The source button was not found in a form.');
        var results = AhValidation.ValidateForm(form, true);
        var isValid = true;
        for (var _i = 0, results_1 = results; _i < results_1.length; _i++) {
            var result = results_1[_i];
            if (!result.IsValid) {
                isValid = false;
                break;
            }
        }
        AhValidation.DisplayResult(form);
        if (isValid) {
            if (args) {
                var hidden = document.createElement('input');
                hidden.setAttribute('type', 'hidden');
                hidden.setAttribute('name', args.Key);
                hidden.setAttribute('value', args.Value);
                form.appendChild(hidden);
            }
            form.submit();
        }
    }
    AhValidation.Submit = Submit;
    /**
     * Decorates an element or elements within a form with validation messages
     * @param elOrForm
     */
    function DisplayResult(elOrForm) {
        var $results;
        if (elOrForm.tagName.toLowerCase() == 'form')
            $results = $('[data-ahvalidationresults]', elOrForm);
        else
            $results = $(elOrForm);
        $results.each(function (idx, el) {
            var $wrap = $(el.parentElement);
            if (!$wrap.hasClass('ahvalidator')) {
                $wrap = $(el).wrap('<span class="ahvalidator" />').parent();
                $wrap.append('<span class="message" />');
            }
            var $message = $('.message', $wrap);
            $wrap.removeClass('error');
            $message.text('');
            var results = JSON.parse(el.getAttribute('data-ahvalidationresults'));
            if (!results || results.IsValid)
                return;
            $wrap.addClass('error');
            var validator = JSON.parse(el.getAttribute('data-ahvalidator'));
            if (!$(el).val() && validator.Required) {
                $message.text('Required');
            }
            else {
                $message.text(results.Message);
            }
        });
    }
    AhValidation.DisplayResult = DisplayResult;
})(AhValidation || (AhValidation = {}));
var AhUrl;
(function (AhUrl) {
    /**
     * Key/Value pairs that represent the hash or querystring
     */
    var UrlPartDic = /** @class */ (function () {
        function UrlPartDic(pairs) {
            this.pairs = pairs || {};
        }
        UrlPartDic.prototype.Get = function (name) {
            for (var key in this.pairs) {
                if (key.toLowerCase() == name.toLowerCase())
                    return this.pairs[key];
            }
            return null;
        };
        UrlPartDic.prototype.Set = function (name, value) {
            for (var key in this.pairs) {
                if (key.toLowerCase() != name.toLowerCase())
                    continue;
                delete this.pairs[key];
            }
            this.pairs[name] = value;
        };
        UrlPartDic.prototype.Remove = function (name) { delete this.pairs[name]; };
        UrlPartDic.prototype.GetPairs = function () { return this.pairs; };
        UrlPartDic.prototype.Format = function () { return FormatPairs(this); };
        return UrlPartDic;
    }());
    AhUrl.UrlPartDic = UrlPartDic;
    /**
    * Formats a sequence of key/value pairs to a URL component.
    * @param format The key/value pairs.
    */
    function FormatPairs(partDic) {
        var pairs = partDic.GetPairs();
        var parts = new Array();
        for (var key in pairs) {
            var part = encodeURIComponent(key);
            var value = pairs[key];
            if (value !== null)
                part += '=' + encodeURIComponent(value);
            parts.push(part);
        }
        return parts.join('&');
    }
    function ParseUrl(param) {
        var url = (param || '').toString();
        var baseUrl = url;
        var hashString = null;
        var queryString = null;
        var hashIndex = url.indexOf('#');
        var queryIndex = url.indexOf('?');
        if (hashIndex >= 0 && hashIndex > queryIndex) {
            hashString = url.substr(hashIndex + 1);
            if (queryString >= 0) {
                queryString = url.substr(queryIndex + 1, hashIndex - queryIndex - 1);
                baseUrl = url.substr(0, queryIndex);
            }
            else {
                baseUrl = url.substr(0, hashIndex);
            }
        }
        else if (queryIndex >= 0) {
            queryString = url.substr(queryIndex + 1);
            baseUrl = url.substr(0, queryIndex);
        }
        /**
         * Parses a query-style URL component into key/value pairs.
         * @param parse The part of the URL to parse into key/value pairs.
         */
        var parse = function (urlPart) {
            var obj = {};
            if (!urlPart)
                return obj;
            var pairs = urlPart.split('&');
            for (var i = 0; i < pairs.length; i++) {
                var data = pairs[i].split('=', 2);
                var key = decodeURIComponent(data[0]);
                var value = data.length > 1 ? decodeURIComponent(data[1]) : null;
                obj[key] = value;
            }
            return obj;
        };
        return {
            BaseUrl: baseUrl,
            Query: new UrlPartDic(parse(queryString)),
            Hash: new UrlPartDic(parse(hashString))
        };
    }
    AhUrl.ParseUrl = ParseUrl;
    ;
    function FormatUrl(param, query, hash) {
        var o;
        if (typeof param === 'string') {
            o = {
                BaseUrl: param || '',
                Query: new UrlPartDic(query || {}),
                Hash: new UrlPartDic(hash || {})
            };
        }
        else {
            o = param;
        }
        var url = o.BaseUrl;
        var queryString = FormatPairs(o.Query);
        var hashString = FormatPairs(o.Hash);
        if (queryString)
            url += '?' + queryString;
        if (hashString)
            url += '#' + hashString;
        return url;
    }
    AhUrl.FormatUrl = FormatUrl;
    ;
    /**
     * Sets the location's hash
     * @param param the url, urldic, or string to set
     * @param updateHistory Value indicating whether to add a history entry.</param>
     */
    function SetHash(param, updateHistory) {
        var updateHistory = updateHistory || false;
        var hash = param.Hash || param;
        var value = (typeof param === 'string') ? param : hash.Format();
        var h = value || '';
        if (h.length > 0 && h[0] == '#')
            h = h.substr(1);
        if (updateHistory) {
            window.location.hash = h;
        }
        else {
            window.location.replace(window.location.href.replace(/(#.*)?$/, '#' + h));
        }
    }
    AhUrl.SetHash = SetHash;
    ;
    /**
         * Sets the location's hash
         * @param param the url, urldic, or string to set
         * @param updateHistory Value indicating whether to add a history entry.
         * @param setLocationIfNeeded if the browser does not support the change (IE9 and earlier), then the location is set, which will cause a reload
         */
    function SetUrl(url, updateHistory, setLocationIfNeeded) {
        var updateHistory = updateHistory || false;
        if (window.history.replaceState) {
            if (updateHistory)
                window.history.pushState({}, document.title, AhUrl.FormatUrl(url));
            else
                window.history.replaceState({}, document.title, AhUrl.FormatUrl(url));
        }
        else {
            var urlStr = AhUrl.FormatUrl(url);
            if (urlStr == window.location.href)
                return;
            var curUrl = AhUrl.ParseUrl(window.location);
            if (curUrl.BaseUrl == url.BaseUrl && curUrl.Query == url.Query) {
                AhUrl.SetHash(url, updateHistory);
            }
            else if (setLocationIfNeeded) {
                console.log('Navigation due to legacy browser support...');
                if (updateHistory)
                    window.location.href = FormatUrl(url);
                else
                    window.location.replace(FormatUrl(url));
            }
        }
    }
    AhUrl.SetUrl = SetUrl;
    ;
    /**
     * Gets the location's hash
     */
    function GetHash() {
        var hash = window.location.hash;
        if (hash && hash[0] == '#')
            return hash.substr(1);
        else
            return hash;
    }
    AhUrl.GetHash = GetHash;
    ;
})(AhUrl || (AhUrl = {}));
var AH;
(function (AH) {
    function EnsureStringOrEmpty(maybeString) {
        if (typeof maybeString === 'string')
            return maybeString + '';
        if (typeof maybeString === undefined || maybeString == null || isNaN(maybeString))
            return '';
        if (maybeString.toString || typeof maybeString.toString === 'function')
            return maybeString.toString();
        return maybeString + '';
    }
    AH.EnsureStringOrEmpty = EnsureStringOrEmpty;
    var TakeIterator = /** @class */ (function () {
        function TakeIterator(source, howMany) {
            this.source = source;
            this.howMany = howMany;
            this.current = null;
            this.taken = 0;
        }
        TakeIterator.prototype.getCurrent = function () {
            return this.current;
        };
        TakeIterator.prototype.moveNext = function () {
            if (this.howMany <= this.taken)
                return false;
            var moveResult = this.source.moveNext();
            if (moveResult) {
                this.current = this.source.getCurrent();
                this.taken += 1;
                return true;
            }
            return false;
        };
        return TakeIterator;
    }());
    var WhereIterator = /** @class */ (function () {
        function WhereIterator(source, predicate) {
            this.source = source;
            this.predicate = predicate;
            this.current = null;
        }
        WhereIterator.prototype.getCurrent = function () {
            return this.current;
        };
        WhereIterator.prototype.moveNext = function () {
            var moveResult = this.source.moveNext();
            while (moveResult) {
                var item = this.source.getCurrent();
                if (this.predicate(item)) {
                    this.current = item;
                    return true;
                }
                moveResult = this.source.moveNext();
            }
            return false;
        };
        return WhereIterator;
    }());
    var SelectIterator = /** @class */ (function () {
        function SelectIterator(source, select) {
            this.source = source;
            this.select = select;
            this.current = null;
            this.index = -1;
        }
        SelectIterator.prototype.getCurrent = function () {
            return this.current;
        };
        SelectIterator.prototype.moveNext = function () {
            var moveResult = this.source.moveNext();
            if (moveResult) {
                this.index++;
                this.current = this.select(this.source.getCurrent(), this.index);
                return true;
            }
            else {
                return false;
            }
        };
        return SelectIterator;
    }());
    var ConcatIterator = /** @class */ (function () {
        function ConcatIterator(originalSource, sources) {
            this.originalSource = originalSource;
            this.sources = sources;
            this.sourceIndex = -1;
            this.currentIterator = null;
        }
        ConcatIterator.prototype.getCurrent = function () {
            return this.currentIterator.getCurrent();
        };
        ConcatIterator.prototype.moveNext = function () {
            while (this.currentIterator == null || !this.currentIterator.moveNext()) {
                if (this.sourceIndex >= this.sources.length)
                    return false;
                if (this.sourceIndex == -1) {
                    this.currentIterator = this.originalSource;
                    this.sourceIndex = 0;
                }
                else {
                    this.currentIterator = this.sources[this.sourceIndex++]();
                }
            }
            return true;
        };
        return ConcatIterator;
    }());
    var ArrayIterator = /** @class */ (function () {
        function ArrayIterator(array) {
            this.array = array;
            this.index = -1;
        }
        ArrayIterator.prototype.getCurrent = function () {
            return this.array[this.index];
        };
        ArrayIterator.prototype.moveNext = function () {
            if (this.index < this.array.length - 1) {
                this.index++;
                return true;
            }
            else {
                return false;
            }
        };
        return ArrayIterator;
    }());
    var JQueryIterator = /** @class */ (function () {
        function JQueryIterator(jq) {
            this.jq = jq;
            this.index = -1;
        }
        JQueryIterator.prototype.getCurrent = function () {
            return $(this.jq[this.index]);
        };
        JQueryIterator.prototype.moveNext = function () {
            if (this.index < this.jq.length - 1) {
                this.index++;
                return true;
            }
            else {
                return false;
            }
        };
        return JQueryIterator;
    }());
    function makeIterable(source) {
        if ($.isArray(source))
            return new DeferredIterableImpl(function () { return new ArrayIterator(source); });
        else if (source instanceof jQuery)
            return new DeferredIterableImpl(function () { return new JQueryIterator(source); });
        else
            return undefined;
    }
    AH.makeIterable = makeIterable;
    var DeferredIterableImpl = /** @class */ (function () {
        function DeferredIterableImpl(createIterator) {
            this.createIterator = createIterator;
        }
        DeferredIterableImpl.prototype.getIterator = function () {
            return this.createIterator();
        };
        DeferredIterableImpl.prototype.where = function (predicate) {
            var _this = this;
            return new DeferredIterableImpl(function () { return new WhereIterator(_this.getIterator(), predicate); });
        };
        DeferredIterableImpl.prototype.select = function (select) {
            var _this = this;
            return new DeferredIterableImpl(function () { return new SelectIterator(_this.getIterator(), select); });
        };
        DeferredIterableImpl.prototype.concat = function () {
            var _this = this;
            var others = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                others[_i] = arguments[_i];
            }
            return new DeferredIterableImpl(function () { return new ConcatIterator(_this.getIterator(), $.map(others, function (i) { return function () { return i.getIterator(); }; })); });
        };
        DeferredIterableImpl.prototype.take = function (howMany) {
            var _this = this;
            return new DeferredIterableImpl(function () { return new TakeIterator(_this.getIterator(), howMany); });
        };
        DeferredIterableImpl.prototype.any = function (predicate) {
            var iterable = predicate ? this.where(predicate) : this;
            var iterator = iterable.getIterator();
            return iterator.moveNext();
        };
        DeferredIterableImpl.prototype.first = function (predicate) {
            var iterable = predicate ? this.where(predicate) : this;
            var iterator = iterable.getIterator();
            if (iterator.moveNext())
                return iterator.getCurrent();
            else
                return null;
        };
        DeferredIterableImpl.prototype.last = function (predicate) {
            var iterable = predicate ? this.where(predicate) : this;
            var iterator = iterable.getIterator();
            var item = null;
            while (iterator.moveNext()) {
                item = iterator.getCurrent();
            }
            return item;
        };
        DeferredIterableImpl.prototype.count = function (predicate) {
            var iterable = predicate ? this.where(predicate) : this;
            var iterator = iterable.getIterator();
            var count = 0;
            while (iterator.moveNext()) {
                count++;
            }
            return count;
        };
        DeferredIterableImpl.prototype.each = function (action) {
            var iterator = this.getIterator();
            var index = 0;
            while (iterator.moveNext()) {
                action(iterator.getCurrent(), index);
                index++;
            }
        };
        DeferredIterableImpl.prototype.toArray = function () {
            var array = new Array();
            var iterator = this.getIterator();
            while (iterator.moveNext()) {
                array.push(iterator.getCurrent());
            }
            return array;
        };
        DeferredIterableImpl.prototype.toObject = function (getName, getValue) {
            var obj = {};
            var iterator = this.getIterator();
            while (iterator.moveNext()) {
                var item = iterator.getCurrent();
                obj[getName(item)] = getValue ? getValue(item) : item;
            }
            return obj;
        };
        return DeferredIterableImpl;
    }());
})(AH || (AH = {}));
//# sourceMappingURL=AH.js.map